/*************************************************** 
  This is a library for our I2C LED Backpacks

  Designed specifically to work with the Adafruit LED Matrix backpacks 
  ----> http://www.adafruit.com/products/
  ----> http://www.adafruit.com/products/

  These displays use I2C to communicate, 2 pins are required to 
  interface. There are multiple selectable I2C addresses. For backpacks
  with 2 Address Select pins: 0x70, 0x71, 0x72 or 0x73. For backpacks
  with 3 Address Select pins: 0x70 thru 0x77

  Adafruit invests time and resources providing this open source code, 
  please support Adafruit and open-source hardware by purchasing 
  products from Adafruit!

  Written by Limor Fried/Ladyada for Adafruit Industries.  
  BSD license, all text above must be included in any redistribution
 ****************************************************/

/*
This code is a modified version for Atmel Studio C environment
and relates to seven segment display only.
Modified by Dirk Rodenhausen (dirk at avr-praxis.de)
*/

#include <stdint.h>
#include <stdbool.h>
#include <avr/pgmspace.h>

#include "SevenSegment_LEDBackpack.h"
#include "TWI_Master.h"


static const uint8_t numbertable[] PROGMEM = { 
	0x3F, /* 0 */
	0x06, /* 1 */
	0x5B, /* 2 */
	0x4F, /* 3 */
	0x66, /* 4 */
	0x6D, /* 5 */
	0x7D, /* 6 */
	0x07, /* 7 */
	0x7F, /* 8 */
	0x6F, /* 9 */
	0x77, /* a */
	0x7C, /* b */
	0x39, /* C */
	0x5E, /* d */
	0x79, /* E */
	0x71, /* F */
	0x00, /* clear */
};

void LEDDriver_SetBrightness(uint8_t b)
{
	if (b > 15) b = 15;
	I2C_Start(displaymodule_addr<<1);
	I2C_Write(HT16K33_CMD_BRIGHTNESS | b);
	I2C_Stop();  
}

void LEDDriver_BlinkRate(uint8_t b)
{
	I2C_Start(displaymodule_addr<<1);
	if (b > 3) b = 0; // turn off if not sure
  
	I2C_Write(HT16K33_CMD_BLINK | HT16K33_BLINK_DISPLAYON | (b << 1)); 
	I2C_Stop();
}


void LEDDriver_SetOscillator(bool osc_on)
{
	
	I2C_Start(displaymodule_addr<<1);
  	
	if (osc_on)
		I2C_Write(HT16K33_CMD_OSCILLATOR | 1);  // turn on oscillator 	
	 else
		I2C_Write(HT16K33_CMD_OSCILLATOR);      // turn off oscillator 	
	 
	I2C_Stop();	 
}

void LEDDriver_Init(void)
{  
	LEDDriver_SetOscillator(true);  
	LEDDriver_BlinkRate(HT16K33_BLINK_OFF);  
	LEDDriver_SetBrightness(15); // max brightness 15 (0..15)
}




void LEDDriver_WriteDisplay(void)
{
	I2C_Start(displaymodule_addr<<1);
	I2C_Write(0x00); // start at address $00

	for (uint8_t i=0; i<8; i++) {
		I2C_Write(displaybuffer[i] & 0xFF);    
		I2C_Write(displaybuffer[i] >> 8);    
	}
	I2C_Stop();  
}

void LEDDriver_DisplayClear(void)
{
	for (uint8_t i=0; i<8; i++) {
		displaybuffer[i] = 0;
	}
}

void LEDDriver_DisplayFill(void)
{
	for (uint8_t i=0; i<8; i++) {
		displaybuffer[i] = 0xFF;
	}
}

void LEDDriver_DrawColon(bool state)
{
	if (state)
		displaybuffer[2] = 0xFF;
	else
		displaybuffer[2] = 0;
}

void LEDDriver_WriteDigitNumber(uint8_t digit, uint8_t number, bool dot)
{
	if (digit > 4) return;
	displaybuffer[digit] = (pgm_read_byte(&numbertable[number]) | (dot << 7));
}

void LEDDriver_DrawDigitDot(uint8_t digit, bool dot)
{
	if (digit > 4) return;
  
	if (dot)
		displaybuffer[digit] |= dot << 7;
	else
		displaybuffer[digit] &= ~(dot << 7);  
}
void LEDDriver_WriteDigitRaw(uint8_t digit, uint8_t bitmask)
{
	if (digit > 4) return;
	displaybuffer[digit] = bitmask;
}

void LEDDriver_DisplayDecimal8(uint8_t d)
{

	LEDDriver_WriteDigitNumber(0, 0, false);
	LEDDriver_WriteDigitNumber(1, (d / 100) % 10, false);
	LEDDriver_WriteDigitNumber(3, (d / 10) % 10, false);
	LEDDriver_WriteDigitNumber(4, d % 10, false);	
		
}

void LEDDriver_DisplayDecimal16(uint16_t d)
{
	
	LEDDriver_WriteDigitNumber(0, (d / 1000), false);
	LEDDriver_WriteDigitNumber(1, (d / 100) % 10, false);
	LEDDriver_WriteDigitNumber(3, (d / 10) % 10, false);
	LEDDriver_WriteDigitNumber(4, d % 10, false);
	
}

void LEDDriver_DisplayHex8(uint8_t d)
{

	LEDDriver_WriteDigitNumber(0, 0, false);
	LEDDriver_WriteDigitNumber(1, 0, false);
	LEDDriver_WriteDigitNumber(3, d >> 4, false);
	LEDDriver_WriteDigitNumber(4, d & 0x0F, false);
	
}

void LEDDriver_DisplayHex16(uint16_t d)
{

	LEDDriver_WriteDigitNumber(0, (uint8_t)(d >> 12) & 0x0F, false);
	LEDDriver_WriteDigitNumber(1, (uint8_t)(d >> 8) & 0x0F, false);
	LEDDriver_WriteDigitNumber(3, (uint8_t)(d >> 4) & 0x0F, false);
	LEDDriver_WriteDigitNumber(4, (uint8_t)d & 0x0F, false);
	
}

